"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Zoom = Zoom;

var _react = _interopRequireWildcard(require("react"));

var _reactNativeReanimated = _interopRequireWildcard(require("react-native-reanimated"));

var _reactNativeGestureHandler = require("react-native-gesture-handler");

var _zoomListContext = require("./zoom-list-context");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function Zoom(props) {
  const {
    minimumZoomScale = 1,
    maximumZoomScale = 8,
    style: propStyle,
    onLayout,
    onZoomBegin,
    onZoomEnd,
    simultaneousGesture
  } = props;
  const zoomListContext = (0, _react.useContext)(_zoomListContext.ZoomListContext);
  const translationX = (0, _reactNativeReanimated.useSharedValue)(0);
  const translationY = (0, _reactNativeReanimated.useSharedValue)(0);
  const originX = (0, _reactNativeReanimated.useSharedValue)(0);
  const originY = (0, _reactNativeReanimated.useSharedValue)(0);
  const scale = (0, _reactNativeReanimated.useSharedValue)(1);
  const isPinching = (0, _reactNativeReanimated.useSharedValue)(false);
  const isZoomed = (0, _reactNativeReanimated.useSharedValue)(false);
  const viewHeight = (0, _reactNativeReanimated.useSharedValue)(0);
  const viewWidth = (0, _reactNativeReanimated.useSharedValue)(0);
  const prevScale = (0, _reactNativeReanimated.useSharedValue)(0);
  const offsetScale = (0, _reactNativeReanimated.useSharedValue)(0);
  const prevTranslationX = (0, _reactNativeReanimated.useSharedValue)(0);
  const prevTranslationY = (0, _reactNativeReanimated.useSharedValue)(0);
  const panTranslateX = (0, _reactNativeReanimated.useSharedValue)(0);
  const panTranslateY = (0, _reactNativeReanimated.useSharedValue)(0);
  const gesture = (0, _react.useMemo)(() => {
    const resetZoomState = () => {
      'worklet'; // reset all state

      translationX.value = (0, _reactNativeReanimated.withTiming)(0);
      translationY.value = (0, _reactNativeReanimated.withTiming)(0);
      scale.value = (0, _reactNativeReanimated.withTiming)(1);
      originX.value = 0;
      originY.value = 0;
      isPinching.value = false;
      prevScale.value = 0;
      prevTranslationX.value = 0;
      prevTranslationY.value = 0;
      panTranslateX.value = 0;
      panTranslateY.value = 0;
    }; // we only activate pan handler when the image is zoomed or user is not pinching


    const pan = _reactNativeGestureHandler.Gesture.Pan().onStart(() => {
      if (isPinching.value || !isZoomed.value) return;
      (0, _reactNativeReanimated.cancelAnimation)(translationX);
      (0, _reactNativeReanimated.cancelAnimation)(translationY);
      (0, _reactNativeReanimated.cancelAnimation)(scale);
      prevTranslationX.value = translationX.value;
      prevTranslationY.value = translationY.value;
    }).onUpdate(e => {
      if (isPinching.value || !isZoomed.value) {
        panTranslateX.value = e.translationX;
        panTranslateY.value = e.translationY;
      } else {
        // imagine what happens to pixels when we zoom in. (they get multiplied by x times scale)
        const maxTranslateX = viewWidth.value / 2 * scale.value - viewWidth.value / 2;
        const minTranslateX = -maxTranslateX;
        const maxTranslateY = viewHeight.value / 2 * scale.value - viewHeight.value / 2;
        const minTranslateY = -maxTranslateY;
        const nextTranslateX = prevTranslationX.value + e.translationX - panTranslateX.value;
        const nextTranslateY = prevTranslationY.value + e.translationY - panTranslateY.value;

        if (nextTranslateX > maxTranslateX) {
          translationX.value = maxTranslateX;
        } else if (nextTranslateX < minTranslateX) {
          translationX.value = minTranslateX;
        } else {
          translationX.value = nextTranslateX;
        }

        if (nextTranslateY > maxTranslateY) {
          translationY.value = maxTranslateY;
        } else if (nextTranslateY < minTranslateY) {
          translationY.value = minTranslateY;
        } else {
          translationY.value = nextTranslateY;
        }
      }
    }).onEnd(() => {
      if (isPinching.value || !isZoomed.value) return;
      panTranslateX.value = 0;
      panTranslateY.value = 0;
    });

    const pinch = _reactNativeGestureHandler.Gesture.Pinch().onStart(() => {
      (0, _reactNativeReanimated.cancelAnimation)(translationX);
      (0, _reactNativeReanimated.cancelAnimation)(translationY);
      (0, _reactNativeReanimated.cancelAnimation)(scale);
      prevScale.value = scale.value;
      offsetScale.value = scale.value;
    }).onUpdate(e => {
      // when pointer is 1 we don't want to translate origin
      if (e.numberOfPointers === 1 && isPinching.value) {
        prevTranslationX.value = translationX.value;
        prevTranslationY.value = translationY.value;
        isPinching.value = false;
      } else if (e.numberOfPointers === 2) {
        const newScale = prevScale.value * e.scale;
        if (newScale < minimumZoomScale || newScale > maximumZoomScale) return;
        scale.value = prevScale.value * e.scale; // reset the origin

        if (!isPinching.value) {
          isPinching.value = true;
          originX.value = e.focalX;
          originY.value = e.focalY;
          prevTranslationX.value = translationX.value;
          prevTranslationY.value = translationY.value;
          offsetScale.value = scale.value;
        }

        if (isPinching.value) {
          // translate the image to the focal point as we're zooming
          translationX.value = prevTranslationX.value + -1 * ((scale.value - offsetScale.value) * (originX.value - viewWidth.value / 2));
          translationY.value = prevTranslationY.value + -1 * ((scale.value - offsetScale.value) * (originY.value - viewHeight.value / 2));
        }
      }
    }).onEnd(() => {
      isPinching.value = false;
      prevTranslationX.value = translationX.value;
      prevTranslationY.value = translationY.value;

      if (scale.value < 1.1) {
        resetZoomState();
      }
    });

    const doubleTap = _reactNativeGestureHandler.Gesture.Tap().onStart(e => {
      // if zoomed in or zoomed out, we want to reset
      if (scale.value !== 1) {
        resetZoomState();
      } else {
        // translate the image to the focal point and zoom
        scale.value = (0, _reactNativeReanimated.withTiming)(maximumZoomScale);
        translationX.value = (0, _reactNativeReanimated.withTiming)(-1 * (maximumZoomScale * (e.x - viewWidth.value / 2)));
        translationY.value = (0, _reactNativeReanimated.withTiming)(-1 * (maximumZoomScale * (e.y - viewHeight.value / 2)));
      }
    }).numberOfTaps(2);

    if (zoomListContext !== null && zoomListContext !== void 0 && zoomListContext.simultaneousPanGestureRef) {
      pan.simultaneousWithExternalGesture(zoomListContext === null || zoomListContext === void 0 ? void 0 : zoomListContext.simultaneousPanGestureRef);
    }

    return _reactNativeGestureHandler.Gesture.Race(doubleTap, simultaneousGesture ? _reactNativeGestureHandler.Gesture.Simultaneous(pan, pinch, simultaneousGesture) : _reactNativeGestureHandler.Gesture.Simultaneous(pan, pinch)); // only add prop dependencies
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [maximumZoomScale, minimumZoomScale, zoomListContext, simultaneousGesture]);
  (0, _reactNativeReanimated.useDerivedValue)(() => {
    if (scale.value > 1 && !isZoomed.value) {
      isZoomed.value = true;
      if (zoomListContext) (0, _reactNativeReanimated.runOnJS)(zoomListContext.onZoomBegin)();
      if (onZoomBegin) (0, _reactNativeReanimated.runOnJS)(onZoomBegin)();
    } else if (scale.value === 1 && isZoomed.value) {
      isZoomed.value = false;
      if (zoomListContext) (0, _reactNativeReanimated.runOnJS)(zoomListContext.onZoomEnd)();
      if (onZoomEnd) (0, _reactNativeReanimated.runOnJS)(onZoomEnd)();
    }
  }, [zoomListContext, onZoomBegin, onZoomEnd]);
  const style = (0, _reactNativeReanimated.useAnimatedStyle)(() => {
    return {
      transform: [{
        translateX: translationX.value
      }, {
        translateY: translationY.value
      }, {
        scale: scale.value
      }]
    };
  }, []);
  const memoizedOnLayout = (0, _react.useCallback)(e => {
    viewHeight.value = e.nativeEvent.layout.height;
    viewWidth.value = e.nativeEvent.layout.width;
    onLayout === null || onLayout === void 0 ? void 0 : onLayout(e);
  }, [viewHeight, viewWidth, onLayout]);
  const memoizedStyle = (0, _react.useMemo)(() => [style, propStyle], [style, propStyle]);
  return /*#__PURE__*/_react.default.createElement(_reactNativeGestureHandler.GestureDetector, {
    gesture: gesture
  }, /*#__PURE__*/_react.default.createElement(_reactNativeReanimated.default.View, _extends({}, props, {
    onLayout: memoizedOnLayout,
    style: memoizedStyle
  })));
}
//# sourceMappingURL=zoom.js.map