function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

import React, { forwardRef, useRef, useState, useMemo } from 'react';
import { Gesture, GestureDetector } from 'react-native-gesture-handler';
import { useAnimatedProps, useSharedValue } from 'react-native-reanimated';
import { ZoomListContext } from './zoom-list-context';
export function createZoomListComponent(ScrollComponent) {
  const ListComponent = /*#__PURE__*/forwardRef((props, ref) => {
    const [scrollEnabled, setScrollEnabled] = useState(true);
    const listRef = useRef(Gesture.Native());
    const contextValues = useMemo(() => ({
      onZoomBegin: () => setScrollEnabled(false),
      onZoomEnd: () => setScrollEnabled(true),
      simultaneousPanGestureRef: listRef.current
    }), []);
    return /*#__PURE__*/React.createElement(ZoomListContext.Provider, {
      value: contextValues
    }, /*#__PURE__*/React.createElement(GestureDetector, {
      gesture: listRef.current
    }, /*#__PURE__*/React.createElement(ScrollComponent, _extends({}, props, {
      scrollEnabled: scrollEnabled,
      ref: ref
    }))));
  });
  return ListComponent;
}
export function createZoomListWithReanimatedComponent(ScrollComponent) {
  const ListComponent = /*#__PURE__*/forwardRef((props, ref) => {
    const scrollEnabled = useSharedValue(true);
    const listRef = useRef(Gesture.Native());
    const contextValues = useMemo(() => ({
      onZoomBegin: () => {
        scrollEnabled.value = false;
      },
      onZoomEnd: () => {
        scrollEnabled.value = true;
      },
      simultaneousPanGestureRef: listRef.current
    }), [scrollEnabled]);
    const animatedProps = useAnimatedProps(() => {
      return {
        scrollEnabled: scrollEnabled.value
      };
    });
    return /*#__PURE__*/React.createElement(ZoomListContext.Provider, {
      value: contextValues
    }, /*#__PURE__*/React.createElement(GestureDetector, {
      gesture: listRef.current
    }, /*#__PURE__*/React.createElement(ScrollComponent, _extends({}, props, {
      animatedProps: animatedProps,
      ref: ref
    }))));
  });
  return ListComponent;
}
//# sourceMappingURL=createZoomListComponent.js.map